package com.kakao.events;

import com.kakao.network.multipart.StringPart;
import com.kakao.util.helper.log.Logger;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * S2 이벤트 모델.
 * @author kevin.kang
 * Created by kevin.kang on 2016. 8. 22..
 */
public final class Event implements Serializable {
    private long timestamp;
    private String from;
    private String to;
    private String action;
    private Map<String, Object> props;
    private boolean adidEnabled;

    static final String TIMESTAMP = "timestamp";
    static final String FROM = "from";
    static final String TO = "to";
    static final String ACTION = "action";
    static final String PROPS = "props";
    static final String EVENTS = "events";
    static final String ADID_ENABLED = "adid_enabled";

    public Event(final long timestamp, final String from, final String to, final String action, final Map<String, Object> props, final boolean adidEnabled) {
        this.timestamp = timestamp;
        this.from = from;
        this.to = to;
        this.action = action;
        this.props = props;
        this.adidEnabled = adidEnabled;
    }

    void setFrom(final String from) {
        this.from = from;
    }

    void setAdidEnabled(boolean adidEnabled) {
        this.adidEnabled = adidEnabled;
    }

    public String getFrom() {
        return from;
    }

    public String getTo() {
        return to;
    }

    public String getAction() {
        return action;
    }

    public Map<String, Object> getProps() {
        return props;
    }

    public long getTimestamp() {
        return timestamp;
    }

    boolean getAdidEnabled() {
        return adidEnabled;
    }


    /**
     * S2 이벤트 모델을 만들기 위한 Builder 클래스.
     */
    public static class Builder {
        private long timestamp = 0;
        private String from = null;
        private String to = null;
        private String action = null;
        private Map<String, Object> props = new HashMap<String, Object>();
        private boolean adidEnabled;

        public Builder setTimestamp(final long timestamp) {
            this.timestamp = timestamp;
            return this;
        }

        public Builder setFrom(final String from) {
            this.from = from;
            return this;
        }

        public Builder setAdidEnabled(final boolean adidEnabled) {
            this.adidEnabled = adidEnabled;
            return this;
        }

        public Builder setTo(final String to) {
            this.to = to;
            return this;
        }

        public Builder setAction(final String action) {
            this.action = action;
            return this;
        }

        public Builder setProps(Map<String, Object> props) {
            if (props != null) {
                this.props = props;
            }
            return this;
        }

        public Event build() {
        return new Event(this);
    }
    }

    private Event(Builder builder) {
        this.timestamp = builder.timestamp;
        this.from = builder.from;
        this.to = builder.to;
        this.action = builder.action;
        this.props = builder.props;
        this.adidEnabled = builder.adidEnabled;
    }

    JSONObject propsToJson() throws JSONException {
        JSONObject propsObject = new JSONObject();

        for (String key : this.props.keySet()) {
            propsObject.put(key, this.props.get(key).toString());
        }
        propsObject.put(ADID_ENABLED, this.adidEnabled ? 1 : 0);
        return propsObject;
    }

    @Override
    public String toString() {
        JSONObject event = new JSONObject();
        try {
            event.put(TIMESTAMP, this.timestamp);
            event.put(FROM, this.from);
            event.put(TO, this.to);
            event.put(ACTION, this.action);
            event.put(PROPS, propsToJson());
            return event.toString();
        } catch (JSONException e) {
            Logger.e(e.toString());
            return event.toString();
        }
    }
}
