package com.kakao.events;

import android.content.Context;

import com.google.android.gms.ads.identifier.AdvertisingIdClient;
import com.kakao.network.callback.ResponseCallback;
import com.kakao.network.tasks.KakaoResultTask;
import com.kakao.network.tasks.KakaoTaskQueue;
import com.kakao.util.KakaoParameterException;
import com.kakao.util.helper.CommonProtocol;
import com.kakao.util.helper.SystemInfo;
import com.kakao.util.helper.Utility;
import com.kakao.util.helper.log.Logger;

import java.util.LinkedList;
import java.util.List;

/**
 * 유저의 특정 이벤트를 수집하는 API
 * @author kevin.kang
 * Created by kevin.kang on 2016. 8. 22..
 */
public class EventsService {

    /**
     * 유저가 보내는 이벤트들을 batching하기 위해 메모리에 저장하고 있는 이벤트 리스트.
     */
    private static List<Event> events = new LinkedList<Event>();

    /**
     * 유저가 지금 현재 batch된 이벤트의 갯수를 확인하고 싶을 때 부른다.
     * @return batch되어 있는 이벤트의 갯수.
     */
    public static int batchSize() {
        return events.size();
    }

    /**
     * 하나의 이벤트를 S2 서버로 전송한다.
     * @param context 현재 애플리케이션의 context
     * @param event 전송할 이벤트 object
     * @param callback 이벤트 전송 요청 결과에 대한 handler
     */
    public static void logSingleEvent(final Context context, final Event event, final ResponseCallback<EventsLogResponse> callback) {
        logEvents(context, event, new LinkedList<Event>(), callback);
    }

    private static void logEvents(final Context context, final Event rootEvent, final List<Event> leafEvents, final ResponseCallback<EventsLogResponse> callback) {
        SystemInfo.initialize(context);
        KakaoTaskQueue.getInstance().addTask(new KakaoResultTask<EventsLogResponse>(callback) {
            @Override
            public EventsLogResponse call() throws Exception {

                if (rootEvent.getFrom() == null) {
                    AdvertisingIdClient.Info idInfo = AdvertisingIdClient.getAdvertisingIdInfo(context);
                    if (idInfo != null) {
                        rootEvent.setFrom(idInfo.getId());
                        rootEvent.setAdidEnabled(!idInfo.isLimitAdTrackingEnabled());
                    }
                }

                EventsLogResponse response = EventsApi.requestLoggingEvents(Utility.getMetadata(context, CommonProtocol.APP_KEY_PROPERTY), rootEvent, leafEvents);

                if (response == null) {
                    throw new KakaoParameterException("There was no response from the request.");
                }

                if (leafEvents.size() > 0) {
                    events.clear();
                }

                return response;
            }
        });
    }

    /**
     * 하나의 이벤트를 이벤트 batch에 추가한다.
     * @param context 현재 애플리케이션의 context
     * @param event batch할 이벤트 object
     * @param callback 이벤트 배치 요청 결과에 대한 handler
     */
    public static void addEventToBatch(final Context context, final Event event, final ResponseCallback<Integer> callback) {
        KakaoTaskQueue.getInstance().addTask(new KakaoResultTask<Integer>(callback) {
            @Override
            public Integer call() throws Exception {
                if (event.getFrom() == null) {
                    AdvertisingIdClient.Info idInfo = AdvertisingIdClient.getAdvertisingIdInfo(context);
                    if (idInfo == null) {
                        throw new IllegalStateException("Failed to get ADID from the device.");
                    }
                    event.setFrom(idInfo.getId());
                    event.setAdidEnabled(!idInfo.isLimitAdTrackingEnabled());
                }
                events.add(event);
                return events.size();
            }
        });
    }

    /**
     * batch된 이벤트들을 S2 서버로 전송한다.
     * @param context 현재 애플리케이션의 context
     * @param callback 이벤트 전송 요청 결과에 대한 handler
     */
    public static void logBatchedEvents(final Context context, final ResponseCallback<EventsLogResponse> callback) {
        logEvents(context, new Event.Builder().build(), events, callback);
    }
}
